"""
wagtail_editorjs
© 2024 Nigel van Keulen, goodadvice.it

A Wagtail integration for the Editor.js block editor.
Includes support for custom blocks, inline tools, and more.
Everything is dynamic and customizable.
"""

__version__ = '1.5.6'
VERSION = tuple([int(v) for v in __version__.split('.')])
